/*			FINGER ACCESS				HTFinger.c
**			=============
** Authors:
**  ARB  Andrew Brooks
**
** History:
**	21 Apr 94   First version (ARB, from HTNews.c by TBL)
*/

#include "HTFinger.h"
#include "HTUtils.h"
#include "tcp.h"
#include "HTML.h"
#include "HTParse.h"
#include "HTFormat.h"

struct _HTStructured 
{
  CONST HTStructuredClass *	isa;
  /* ... */
};

/* #define TRACE 1 */

#define FINGER_PORT 79		/* See rfc742 */

/*	Module-wide variables
*/
PRIVATE int s;					/* Socket for FingerHost */
PRIVATE HTStructured * target;			/* The output sink */
PRIVATE HTStructuredClass targetClass;		/* Copy of fn addresses */

#define PUTC(c) (*targetClass.put_character)(target, c)
#define PUTS(s) (*targetClass.put_string)(target, s)
#define START(e) (*targetClass.start_element)(target, e, 0, 0)
#define END(e) (*targetClass.end_element)(target, e)


/*	Initialisation for this module
**	------------------------------
*/
PRIVATE BOOL initialized = NO;
PRIVATE BOOL initialize NOARGS
{
  s = -1;		/* Disconnected */
  return YES;
}



/*	Send Finger Command line to remote host & Check Response
**	--------------------------------------------------------
**
** On entry,
**	command	points to the command to be sent, including CRLF, or is null
**		pointer if no command to be sent.
** On exit,
**	Negative status indicates transmission error, socket closed.
**	Positive status is a Finger status.
*/


PRIVATE int response ARGS1(CONST char *,command)
{
  int status;
  int length = strlen(command);
  int result;    
  int ch;

  if (length==0) return(-1);
  if (TRACE) 
    fprintf(stderr, "Finger command to be sent: %s", command);
  status = NETWRITE(s, command, length);
  if (status<0)
    {
      if (TRACE) fprintf(stderr,
                         "HTFinger: Unable to send command. Disconnecting.\n");
      NETCLOSE(s);
      s = -1;
      return status;
    } /* if bad status */
  
  if (TRACE)
	fprintf(stderr,"Reading finger information\n");
  HTInitInput(s);
  START(HTML_PRE);
  while ((ch = HTGetCharacter()) != ((char)EOF)) PUTC(ch);
  END(HTML_PRE);
  if (TRACE)
	fprintf(stderr,"Closing net\n");
  NETCLOSE(s);
  s = -1;
  if (TRACE)
	fprintf(stderr,"exiting\n");
  return(0);
}


/*	Abort the connection					abort_socket
**	--------------------
*/
PRIVATE void abort_socket NOARGS
{
    if (TRACE) fprintf(stderr,
	    "HTFinger: EOF on read, closing socket %d\n", s);
    NETCLOSE(s);	/* End of file, close socket */
    PUTS(NETWORK_CONNECTION_LOST);
    PUTC('\n');
    s = -1;		/* End of file on response */
    return;
}


/*		Load by name					HTLoadFinger
**		============
*/
PUBLIC int HTLoadFinger ARGS4(
	CONST char *,		arg,
	HTParentAnchor *,	anAnchor,
	HTFormat,		format_out,
	HTStream*,		stream)
{
  char *at, *username, *sitename;       /* Fields extracted from URL */
  char str[1024];			/* Buffer */
  int status;				/* tcp return */
  int retries;			/* A count of how hard we have tried */ 
  
  if (TRACE) 
    fprintf(stderr, "HTFinger: Looking for %s\n", arg);
  
  if (!initialized) 
    initialized = initialize();
  if (!initialized) 
    {
      HTProgress (NO_FINGER_CONNECTION);
      return HT_NOT_LOADED;	/* FAIL */
    }
    
  {
    CONST char * p1=arg;
    
    /*	We will ask for the document, omitting the host name & anchor.
     **
     **	Syntax of address is
     **		xxx@yyy			User xxx at site yyy (xxx is optional).
     */        
    if (!strncasecomp (arg, "finger:", 7))
      p1 = arg + 7;  /* Skip "finger:" prefix */

    at = strchr(p1, '@');
    if (at == NULL)
      {
        HTProgress (NO_FINGER_DATA_IN_URL);
        return HT_NOT_LOADED;			/* Ignore if no name */
      }

    username = p1;
    sitename = at+1;
    *at = '\0';
    if (strlen(sitename)==0)
      {
        HTProgress (NO_FINGER_SITENAME);
        return HT_NOT_LOADED;			/* Ignore if no name */
      }
    
      /* *p++ = CR; *p++ = LF; *p++ = 0; */
  } /* scope of p1 */
  
  if (!*arg) 
    {
      HTProgress (COULDNT_LOAD_DATA);
      return HT_NOT_LOADED;			/* Ignore if no name */
    }
  
  /*	Make a hypertext object with an anchor list. */       
  target = HTML_new(anAnchor, format_out, stream);
  targetClass = *target->isa;	/* Copy routine entry points */
  
    	
  /*	Now, let's get a stream setup up from the FingerHost: */       
  /* CONNECTING to finger host */
  sprintf (str, "lose://%s/", sitename);
  if (TRACE)
    fprintf (stderr, "Finger: doing HTDoConnect on '%s'\n", str);
  status = HTDoConnect (str, "Finger", FINGER_PORT, &s);
  if (TRACE)
    fprintf (stderr, "Finger: Done DoConnect; status %d\n", status);
  if (status == HT_INTERRUPTED)
    {
      /* Interrupt cleanly. */
      if (TRACE)
        fprintf (stderr,
               "Finger: Interrupted on connect; recovering cleanly.\n");
      HTProgress (CONNECTION_INTERRUPTED);
      (*targetClass.handle_interrupt)(target);
      return HT_INTERRUPTED;
    }
  if (status < 0)
    {
      char message[256];
      NETCLOSE(s);
      s = -1;
      if (TRACE) 
        fprintf(stderr, "HTFinger: Unable to connect to finger host.\n");
      HTProgress (NO_FINGER_ACCESS);
      sprintf(message, NO_FINGER_ACCESS2,
              sitename);
      PUTS(message);
      (*targetClass.end_document)(target);
      (*targetClass.free)(target);
      return HT_LOADED;
    } 
  else 
    {
      if (TRACE) fprintf(stderr, "HTFinger: Connected to finger host %s.\n",
                         sitename);
      HTInitInput(s);		/* set up buffering */
      sprintf(str, "%s%c%c", username, CR, LF);
      START(HTML_TITLE);
      PUTS(arg);
      END(HTML_TITLE);
      if (response(str) != 0) 
        {
          NETCLOSE(s);
          s = -1;
          /* Couldn't get it. */
          START(HTML_TITLE);
          PUTS(COULDNT_RETR_INFO_TITLE);
          END(HTML_TITLE);
          PUTS(COULDNT_RETR_INFO_MSG);
          (*targetClass.end_document)(target);
          (*targetClass.free)(target);
          return HT_LOADED;
        }
    }
      
  (*targetClass.end_document)(target);
  (*targetClass.free)(target);
  return HT_LOADED;
}

PUBLIC HTProtocol HTFinger = { "finger", HTLoadFinger, NULL };
